@@ -16,19 +16,83 @@
 #include <string.h>
 #include <es.h>
 #include <es/exception.h>
+#include "apic.h"
 #include "core.h"
 
+// #define VERBOSE
+
 extern "C" void _exit(int i);
 
-Ref Core::numCores(0);
-Core* Core::coreTable[Core::MaxCore];
+namespace
+{
+    // The NullPic class is only used until the Pic class object for 8259 or
+    // the Apic class object for APIC are constructed.
+    class NullPic : public IPic
+    {
+        Ref ref;
+    public:
+        void startup(unsigned int irq) {}
+        void shutdown(unsigned int irq) {}
+        void enable(unsigned int irq) {}
+        void disable(unsigned int irq) {}
+        bool ack(unsigned int irq) { return false; }
+        void end(unsigned int irq) {}
+        void setAffinity(unsigned int irq, unsigned int mask) {}
+        unsigned int splIdle() { return 0; }
+        unsigned int splLo() { return 0; }
+        unsigned int splHi() { return 0; }
+        void splX(unsigned int x) {}
+        bool queryInterface(const Guid& riid, void** objectPtr)
+        {
+            if (riid == IID_IPic)
+            {
+                *objectPtr = static_cast<IPic*>(this);
+            }
+            else if (riid == IID_IInterface)
+            {
+                *objectPtr = static_cast<IPic*>(this);
+            }
+            else
+            {
+                *objectPtr = NULL;
+                return false;
+            }
+            static_cast<IInterface*>(*objectPtr)->addRef();
+            return true;
+        }
+        unsigned int addRef(void)
+        {
+            return ref.addRef();
+        }
+        unsigned int release(void)
+        {
+            unsigned int count = ref.release();
+            if (count == 0)
+            {
+                delete this;
+                return 0;
+            }
+            return count;
+        }
+    };
+
+    NullPic nullPic __attribute__ ((init_priority (101)));
+
+    struct Frame
+    {
+        Frame* prev;
+        void*  pc;
+    };
+}
+
+Core* Core::coreTable[Core::CORE_MAX];
 bool Core::fxsr;
 bool Core::sse;
 Segdesc Core::idt[256] __attribute__ ((aligned (16)));
 SegdescLoc Core::idtLoc(sizeof idt - 1, idt);
-SpinLock Core::spinLock;
+Lock Core::spinLock;
 ICallback* Core::exceptionHandlers[255];
-IPic* Core::pic;
+IPic* Core::pic = &nullPic;
 
 extern "C"
 {
@@ -352,30 +416,35 @@ static void settr(register u16 sel)
 }
 
 Core::
-Core(Sched* sched, void* stack, unsigned stackSize, Tss* tss) :
+Core(Sched* sched) :
     sched(sched),
     currentProc(0),
     current(0),
     currentFPU(0),
-    yieldable(true),
-    label(stack, stackSize, reschedule, this),
-    tss(tss),
+    freeze(0),
     gdtLoc(sizeof gdt - 1, gdt),
     tcb(0)
 {
-    ASSERT((unsigned long) this % 16 == 0);
-    ASSERT((unsigned long) tss % 256 == 0);
+    ASSERT(reinterpret_cast<unsigned long>(this) % 16 == 0);
 
-    id = numCores.addRef() - 1;
-    ASSERT(id < MaxCore);
+    u8* ptr = reinterpret_cast<u8*>((reinterpret_cast<unsigned long>(this) & ~0xff) - 256);
+
+    tss = reinterpret_cast<Tss*>(ptr);
+    ASSERT(reinterpret_cast<unsigned long>(tss) % 256 == 0);
+
+    stack = ptr + 256 + ((sizeof(Core) + 15) & ~15);
+    *(int*) stack = 0xa5a5a5a5;
+    unsigned stackSize = CORE_SIZE - 256 - ((sizeof(Core) + 15) & ~15);
+    label.init(stack, stackSize, reschedule, this);
+
+    id = Sched::numCores - 1;
     coreTable[id] = this;
 
-    // Reserve a page for TCB that can be read from the user level.
-    tcb = (Tcb*) Mmu::getPointer(0x11000 + (id * sizeof(Tcb)));
-    memset(tcb, 0, sizeof(Tcb));
+    esReport("Core %d at %p:%p:%p %p:%d\n", id, this, ptr, tss, stack, stackSize);
 
-    ktcb = (Tcb*) Mmu::getPointer(0x12000 + (id * sizeof(Tcb)));
-    memset(ktcb, 0, sizeof(Tcb));
+    // Reserve a page for TCBs that can be read from the user level.
+    tcb = reinterpret_cast<Tcb*>(0x80011000 + id * sizeof(Tcb));
+    memset(tcb, 0, sizeof(Tcb));
 
     memset(tss, 0, sizeof(Tss));
     tss->ss0 = tss->ds = tss->es = tss->fs = tss->ss = KDATASEL;
@@ -399,7 +468,7 @@ Core(Sched* sched, void* stack, unsigned stackSize, Tss* tss) :
     gdt[4].init(0x30000, 0xffff, Segdesc::SEGP | Segdesc::SEGDATA | Segdesc::SEGW);
 
     // KTCBSEL
-    gdt[5].init(0x80000000 + 0x12000 + id * sizeof(Tcb),
+    gdt[5].init(reinterpret_cast<u32>(&ktcb),
                 sizeof(tcb) - 1, Segdesc::SEGD | Segdesc::SEGP | Segdesc::SEGDATA);
 
     // TSSSEL
@@ -417,16 +486,42 @@ Core(Sched* sched, void* stack, unsigned stackSize, Tss* tss) :
 
     if (id == 0)
     {
-        u32 exceptionAddr = (u32) exceptionTable;
-        for (Segdesc* desc = idt; desc < &idt[256]; ++desc, exceptionAddr += 16)
+        u32 exceptionAddr = reinterpret_cast<u32>(exceptionTable);
+        for (int vec = 0; vec < 256; ++vec, exceptionAddr += 16)
         {
-            desc->setExceptionHandler(KCODESEL, (void (*)()) exceptionAddr);
+            Segdesc* desc = &idt[vec];
+            switch (vec)
+            {
+            case NO_DE:     // Divide Error
+            case NO_DB:     // Debug
+            case NO_BP:     // Breakpoint
+            case NO_OF:     // Overflow
+            case NO_BR:     // BOUND Range Exceeded
+            case NO_UD:     // Invalid Opcode (UnDefined Opcode)
+            case NO_NM:     // Device Not Available (No Math Coprocessor)
+            case NO_MF386:  // CoProcessor Segment Overrun
+            case NO_TS:     // Invalid TSS
+            case NO_NP:     // Segment Not Present
+            case NO_SS:     // Stack Segment Fault
+            case NO_GP:     // General Protection
+            case NO_PF:     // Page Fault
+            case NO_MF:     // Floating-Point Error (Math Fault)
+            case NO_AC:     // Alignment Check
+            case NO_XF:     // SIMD Floating-Point Exception
+                desc->setInterruptHandler(KCODESEL, exceptionAddr);
+                break;
+            case 65:        // System call
+            case 66:        // Upcall
+                desc->setInterruptHandler(KCODESEL, exceptionAddr);
+                desc->setDPL(3);
+                break;
+            default:
+                desc->setInterruptHandler(KCODESEL, exceptionAddr);
+                break;
+            }
         }
     }
 
-    idt[65].setDPL(3);  // for system call
-    idt[66].setDPL(3);  // for upcall
-
     gdtLoc.loadGDT();
     idtLoc.loadIDT();
     settr(TSSSEL);
@@ -443,31 +538,34 @@ Core(Sched* sched, void* stack, unsigned stackSize, Tss* tss) :
     fxsr = (edx & (1u << 24)) ? true : false;
     sse = (edx & (1u << 25)) ? true : false;
     initFPU();
-
-    if (!pic)
-    {
-        pic = new Pic();
-    }
 }
 
 Core* Core::
 getCurrentCore()
 {
-    return coreTable[0];
+    return coreTable[Apic::getLocalApicID()];   // XXX The Local APIC IDs need not to be consecutive.
 }
 
 void Core::
 reschedule(void* param)
 {
-    unsigned x = Thread::splHi();
+    unsigned x = splHi();
     Core* core = getCurrentCore();
     for (;;)
     {
         core->label.set();
-        Thread* current = (Thread*) core->current;
+        Thread* current = core->current;
         if (current)
         {
-            current->lock();
+            // Note the current has been locked when rescheduled by
+            // Rendezvous::sleep().
+            current->tryLock();
+            core->current = 0;
+#ifdef VERBOSE
+            esReport("[%d:%p] Core::reschedule in. current = %p %p %p\n",
+                     core->id, &param, current, current ? current->label.esp : 0);
+#endif
+            ASSERT(current->core == core);
             switch (current->getState())
             {
               case IThread::TERMINATED:
@@ -479,7 +577,6 @@ reschedule(void* param)
                 current->release();
                 break;
               case IThread::RUNNING:
-                current->state = IThread::RUNNABLE;
                 current->setRun();
                 // FALL THROUGH
               default:
@@ -488,14 +585,16 @@ reschedule(void* param)
                     current->xreg.save();
                     disableFPU();
                 }
+                ASSERT(current->label.isSane());
+                current->core = 0;
                 current->unlock();
                 break;
             }
-            core->current = 0;
         }
         core->current = current = core->sched->selectThread();
+        ASSERT(core->checkStack());
         core->currentFPU = 0;
-        core->ktcb->tcb = current->ktcb;
+        core->ktcb.tcb = current->ktcb;
 
         UpcallRecord* record = current->upcallList.getLast();
         if (!record)
@@ -515,12 +614,14 @@ reschedule(void* param)
         core->tss->sp0 = current->sp0;
 
 #ifdef VERBOSE
-        esReport("reschedule: %d %p %p\n", core->id, current, current->label.eip);
+        esReport("[%d:%p] Core::reschedule out. current = %p: %p %p\n",
+                 core->id, &param, current, current ? current->label.esp : 0);
 #endif
 
+        ASSERT(current->label.isSane());
         current->label.jump();
     }
-    Thread::splX(x);
+    splX(x);
     // NOT REACHED HERE
 }
 
@@ -578,13 +679,23 @@ static void kernelFault()
 void Core::
 dispatchException(Ureg* ureg)
 {
+    unsigned x = splHi();
     Core* core = getCurrentCore();
+    Thread* current = core->current;
+    Process* process = core->currentProc;
+
+#ifdef VERBOSE
+    esReport("[%d:%p]: dispatchException(%p:%d) at %p\n",
+              core->id, (u8*) &ureg + sizeof(Ureg) - 8,
+              ureg, ureg->trap, ureg->eip);
+#endif
+
     switch (ureg->trap)
     {
       case NO_NM:   // Device Not Available (No Math Coprocessor)
-        if (core->current)
+        if (current)
         {
-            core->currentFPU = core->current;
+            core->currentFPU = current;
             enableFPU();
             core->current->xreg.restore();
         }
@@ -596,40 +707,53 @@ dispatchException(Ureg* ureg)
             "movl   %%cr2, %0\n"
             : "=r"(cr2));
 #endif // __i386__
-        if (core->currentProc)
+        if (process)
         {
+            // splLo();
+
             int rc;
             if (ureg->error & Page::PTEVALID)
             {
-                rc = core->currentProc->protectionFault((void*) cr2, ureg->error);
+                rc = process->protectionFault((void*) cr2, ureg->error);
             }
             else
             {
-                rc = core->currentProc->validityFault((void*) cr2, ureg->error);
+                rc = process->validityFault((void*) cr2, ureg->error);
             }
             if (0 <= rc)
             {
                 break;
             }
-            if (ureg->cs == KCODESEL && core->currentProc->isValid((void*) cr2, 1))
+            if (ureg->cs == KCODESEL && process->isValid((void*) cr2, 1))
             {
                 // Kernel page fault: Create a stack frame as if kernelFault() is
                 // called. By using the -fnon-call-exceptions g++ compiler option,
                 // the function incurred a kernel page fault will receive an
                 // EFFALT system exception.
-                if (core->currentProc->log)
+                if (process->log)
                 {
-                    esReport("[kernel page fault at %p]", cr2);
+                    esReport("[kernel page fault at %p]\n", cr2);
+                    Frame* frame = (Frame*) (&ureg - 2);
+                    while (frame)
+                    {
+                        esReport("%p %p\n", frame->pc, frame->prev);
+                        frame = frame->prev;
+                    }
                 }
                 Ureg* exc = (Ureg*) ((char*) ureg - sizeof(u32));
                 memmove(exc, ureg, sizeof(Ureg) - 8);
                 u32 ret = exc->eip;
                 exc->esp = ret;     // Note &exc->esp is the new esp address after iret.
                 exc->eip = (u32) kernelFault;
+
+                ASSERT(!current || current->checkStack());
+
+                splX(x);
                 exc->load();
                 break;
             }
         }
+        esReport("Kernel panic [%d]\n", core->id);
         ureg->dump();
         esPanic(__FILE__, __LINE__, "Failed Core::dispatchException: %u @ %x cr2:%x\n", ureg->trap, ureg->eip, cr2);
         break;
@@ -638,19 +762,21 @@ dispatchException(Ureg* ureg)
         // ureg->ecx: array of parameters
         // ureg->edx: method number
         // ureg->esi: base
-        if (core->currentProc)
+        if (process)
         {
-            // esReport("[%p]::dispatchException: %u @ %x\n", core->currentProc, ureg->trap, ureg->eip);
+            splLo();
+
+            // esReport("[%p]::dispatchException: %u @ %x\n", process, ureg->trap, ureg->eip);
             int errorCode(0);
             long long result;
 
-            core->current->param = ureg;
+            current->param = ureg;
             try
             {
                 va_list param(reinterpret_cast<va_list>(ureg->ecx));
 
                 ureg->ecx = 0;  // To indicate this is not an exception call
-                result = core->currentProc->systemCall(
+                result = process->systemCall(
                     reinterpret_cast<void**>(ureg->eax),
                     ureg->edx,
                     param,
@@ -678,7 +804,7 @@ dispatchException(Ureg* ureg)
             ureg->eax = (u32) result;
             ureg->ecx = (u32) errorCode;
             ureg->edx = (u32) (result >> 32);
-            core->current->testCancel();
+            current->testCancel();
             break;
         }
         esPanic(__FILE__, __LINE__, "Failed Core::dispatchException: %u @ %x\n", ureg->trap, ureg->eip);
@@ -686,39 +812,51 @@ dispatchException(Ureg* ureg)
       case 66:  // upcall interface
         if (core->currentProc)
         {
-            // esReport("[%p]::dispatchException: %u @ %x\n", core->currentProc, ureg->trap, ureg->eip);
+            splLo();
+
             ureg->ecx = 0;  // To indicate this is not an exception call
-            core->current->param = ureg;
-            core->currentProc->returnFromUpcall(ureg);
-            core->current->testCancel();
+            current->param = ureg;
+            process->returnFromUpcall(ureg);
+            current->testCancel();
             break;
         }
         esPanic(__FILE__, __LINE__, "Failed Core::dispatchException: %u @ %x\n", ureg->trap, ureg->eip);
         break;
       default:
-        ICallback* callback = core->exceptionHandlers[ureg->trap];
         if (32 <= ureg->trap)
         {
-            core->yieldable = false;
+            core->freeze.increment();
+            core->current = 0;
             int irq = ureg->trap - 32;
+
             if (pic->ack(irq))
             {
+                // x = splLo();    // Enable interrupts
+                ICallback* callback = core->exceptionHandlers[ureg->trap];
                 if (callback)
                 {
                     callback->invoke(irq);
                 }
                 pic->end(irq);
+                // splX(x);
             }
-            core->yieldable = true;
+
+            core->current = current;
+            ASSERT(!current || current->checkStack());
+            core->freeze.decrement();
             Thread::reschedule();
         }
         else
         {
+            esReport("Kernel panic [%d]\n", core->id);
             ureg->dump();
             esPanic(__FILE__, __LINE__, "Core::dispatchException: %u @ %x\n", ureg->trap, ureg->eip);
         }
         break;
     }
+
+    ASSERT(!current || current->checkStack());
+    splX(x);
 }
 
 long Core::
@@ -728,7 +866,7 @@ registerExceptionHandler(u8 exceptionNumber, ICallback* callback)
     {
         return -1;
     }
-    unsigned x = Thread::splHi();
+    unsigned x = splHi();
     spinLock.lock();
     ICallback* old = exceptionHandlers[exceptionNumber];
     exceptionHandlers[exceptionNumber] = callback;
@@ -737,7 +875,7 @@ registerExceptionHandler(u8 exceptionNumber, ICallback* callback)
         pic->startup(exceptionNumber - 32);
     }
     spinLock.unlock();
-    Thread::splX(x);
+    splX(x);
     callback->addRef();
     if (old)
     {
@@ -750,7 +888,7 @@ long Core::
 unregisterExceptionHandler(u8 exceptionNumber, ICallback* callback)
 {
     long rc;
-    unsigned x = Thread::splHi();
+    unsigned x = splHi();
     spinLock.lock();
     ICallback* old = exceptionHandlers[exceptionNumber];
     if (!callback || old == callback)
@@ -767,7 +905,7 @@ unregisterExceptionHandler(u8 exceptionNumber, ICallback* callback)
         pic->shutdown(exceptionNumber - 32);
     }
     spinLock.unlock();
-    Thread::splX(x);
+    splX(x);
     if (old)
     {
         old->release();
@@ -778,7 +916,7 @@ unregisterExceptionHandler(u8 exceptionNumber, ICallback* callback)
 void Core::
 shutdown()
 {
-    unsigned x = Thread::splHi();
+    unsigned x = splHi();
     // Restore default kernel memory map
     u32* table = static_cast<u32*>(Mmu::getPointer(0x10000));
     for (int i = 0; i < 512; ++i)
@@ -791,5 +929,32 @@ shutdown()
         ::: "%eax");
 
     ((int (*)()) (0x30000 + (*(u16*) (0x30000 + 124))))();
-    Thread::splX(x);
+    splX(x);
+}
+
+
+bool Core::
+checkStack()
+{
+    return *(int*) stack == 0xa5a5a5a5;
+}
+
+// Allocates memory for Core, Core stack, and Core TSS at the same time.
+void* Core::
+operator new(size_t size) throw(std::bad_alloc)
+{
+    u8* ptr = reinterpret_cast<u8*>(0x80030000);
+    int n = Sched::numCores.addRef() - 1;
+    ASSERT(n < CORE_MAX);
+    if (CORE_MAX <= n)
+    {
+        throw std::bad_alloc();
+    }
+    ptr += CORE_SIZE * n;
+    return ptr + 256;
+}
+
+void Core::
+operator delete(void*) throw()
+{
 }
