@@ -25,6 +25,7 @@
 #include "8042.h"
 #include "8237a.h"
 #include "8254.h"
+#include "apic.h"
 #include "ataController.h"
 #include "cache.h"
 #include "cga.h"
@@ -34,6 +35,7 @@
 #include "fdc.h"
 #include "heap.h"
 #include "interfaceStore.h"
+#include "mps.h"
 #include "partition.h"
 #include "rtc.h"
 #include "sb16.h"
@@ -49,15 +51,15 @@ namespace
     IClassStore*    classStore;
     InterfaceStore* interfaceStore;
     Sched*          sched;
-    unsigned        coreStack[4096] __attribute__ ((aligned (16)));
-    Tss             coreTss __attribute__ ((aligned (256)));
-    unsigned        mainStack[4096] __attribute__ ((aligned (16)));
     Pit*            pit;
     IRtc*           rtc;
     IStream*        reportStream;
     IClassFactory*  alarmFactory;
     Dmac*           master;
     Dmac*           slave;
+    Pic*            pic;
+    Mps*            mps;
+    Apic*           apic;
 };
 
 const int Page::SIZE = 4096;
@@ -118,6 +120,19 @@ static void initArena()
     free(0);    // Just to link malloc.cpp
 }
 
+static void initAP(...)
+{
+    apic->enableLocalApic();
+    apic->splHi();
+    apic->setTimer(67, 1000);
+    Core* core = new Core(sched);
+    apic->started();
+    core->start();
+    // NOT REACHED HERE
+}
+
+extern int main(int, char* []);
+
 int esInit(IInterface** nameSpace)
 {
     if (root)
@@ -131,25 +146,69 @@ int esInit(IInterface** nameSpace)
 
     Cga* cga = new Cga;
     reportStream = cga;
+#if 1
+    int port = ((u16*) 0x400)[0];
+    if (port)
+    {
+        Uart* uart = new Uart(port);
+        if (uart)
+        {
+            reportStream = uart;
+        }
+    }
+#endif
+
+    // Initialize 8259 anyways.
+    pic = new Pic();
 
     // Initialize the page table
     initArena();
     ASSERT(PageTable::pageSet);
 
+    // Initialize RTC
+    rtc = new Rtc;
+
     // Create the thread scheduler
     sched = new Sched;
 
     // Initialize the current core
-    Core* core = new Core(sched, coreStack, sizeof coreStack, &coreTss);
+    Core* core = new Core(sched);
+
+    mps = new Mps;
+    if (!mps->getFloatingPointerStructure())
+    {
+        Core::pic = pic;
+    }
+    else
+    {
+        // Startup APs
+        u32 hltAP = 0x30000 + *(u16*) (0x30000 + 138);
+        u32 startAP = 0x30000 + *(u16*) (0x30000 + 126);
+        *(u32*) (0x30000 + 132) = (u32) initAP;
+
+        esReport("Startap: %x\n", startAP);
+        esReport("Halt: %x\n", hltAP);
+
+        apic = new Apic(mps);
+        apic->busFreq();
+        Core::pic = apic;
+
+        Core::registerExceptionHandler(67, sched);
+        apic->setTimer(67, 1000);
+
+        apic->startup(hltAP, startAP);
+    }
 
     // Create the default thread (stack top: 0x80010000)
     Thread* thread = new Thread(0, 0, IThread::Normal,
                                 (void*) 0x80009000, 0x80010000 - 0x80009000);
     thread->state = IThread::RUNNING;
     thread->sched = sched;
+    thread->func = (void* (*)(void*)) main;
+    thread->core = core;
     core->current = thread;
+    core->ktcb.tcb = thread->ktcb;
 
-    rtc = new Rtc;
     pit = new Pit(1000);
 
     root = new Context;
@@ -201,22 +260,10 @@ int esInit(IInterface** nameSpace)
     IClassFactory* partitionFactory = new(ClassFactory<PartitionContext>);
     classStore->add(CLSID_Partition, partitionFactory);
 
-#if 1
-    int port = ((u16*) 0x400)[0];
-    if (port)
-    {
-        Uart* uart = new Uart(port);
-        if (uart)
-        {
-            reportStream = uart;
-        }
-    }
-#endif
-
     slave = new Dmac(0x00, 0x80, 0);
     master = new Dmac(0xc0, 0x88, 1);
 
-    __asm__ __volatile__ ("sti\n");
+    Core::pic->splLo();
 
     root->bind("device/beep", static_cast<IBeep*>(pit));
 
@@ -292,7 +339,6 @@ void esPanic(const char* file, int line, const char* msg, ...)
     esReport(" in \"%s\" on line %d.\n", file, line);
 
     Frame* frame = (Frame*) (&file - 2);
-    esReport("%p:\n", frame);
     while (frame)
     {
         esReport("%p %p\n", frame->pc, frame->prev);
@@ -310,8 +356,14 @@ void esPanic(const char* file, int line, const char* msg, ...)
 
 int esReportv(const char* spec, va_list list)
 {
+    static Lock lock;
+    unsigned x = Core::splHi();
+    lock.lock();
     Formatter formatter(reportStream);
-    return formatter.format(spec, list);
+    int len = formatter.format(spec, list);
+    lock.unlock();
+    Core::splX(x);
+    return len;
 }
 
 ICurrentProcess* esCurrentProcess()
